<?php

namespace HCM\FPA\Calculos\Folha;

use HCM\FPA\Calculos\Geral\Calculo;
use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\Util\FPA;
use HCM\Util\TkDI;
use HCM\Util\MsgCalculo;
use HCM\FPA\Calculos\Geral\MemoriaCalculo;
use HCM\FPA\Entity\FpatipoincideEntity;
use HCM\FPA\Entity\FpaProcessoEntity;
use HCM\FPA\Entity\FpaPtregraEntity;
use HCM\FPA\Entity\FpaValorreferenciamEntity;
use HCM\Geral\Entity\GpesitufuncmEntity;
use HCM\Geral\Entity\GpevinculohEntity;
use HCM\FPA\Entity\FpaParamfolhahEntity;
use HCM\FPA\Entity\FpatipomovimentEntity;
use HCM\FPA\Entity\FpaFormcalculoEntity;
use HCM\FPA\Entity\FpaformrescisaoEntity;
use HCM\FPA\Entity\FpatipoassoceveEntity;
use HCM\FPA\Entity\FpaitecalcfolhaEntity;
use HCM\FPA\Entity\FpatipocalculoEntity;
use HCM\FPA\Entity\FpacalculofolhaEntity;
use Teknisa\Model\Gestaopessoas\Repositories\FpaValorreferenciam;
use HCM\FPA\Calculos\Geral\FGTS;
use HCM\FPA\Calculos\Geral\INSS;
use HCM\FPA\Calculos\Geral\IRRF;

class CalculoFolha extends Calculo {

    public function calculaFolha($vinculo, $tp_calc_mov, $contador){

        $calcula = $this->_inicializaVariaveis($vinculo, $tp_calc_mov);
        
        if(!$calcula){
            return;
        }
        
        if ($tp_calc_mov->retornaItensCalculo($vinculo) === false) {
            return;
        }
        try {
            $obj_tp_calc_mov = current($tp_calc_mov->retornaPosicaoTpMov($tp_calc_mov->retornaTpMovPrincipal()));
            $this->_itens_calculo_principal =  $obj_tp_calc_mov->getInstancia();
            $this->_itens_calculo_principal->setNrTipoCalculo($this->_nr_tipo_calculo);
        } catch(Exception $e) {
            $message = $e->getMessage();
            $message .= ' - CALCULO.PHP';
            MsgCalculo::getInstance()->adicionaMensagem($e->getMessage());
        }

        $liquido = $this->_controleLiquido();
        
        if ($liquido == true) {
            MsgCalculo::getInstance()->adicionaMensagem('Vinculo possui liquido.', MsgCalculo::ERRO);
            return;
        }
        
        $consolidaINSScalculo = $this->entityManager->getRepository(Repositories::FPA_VALORREFERENCIAM)->buscavalor($this->_nrorg, $this->_nrOrgPadrao, $this->_mes_competencia, 'ID_CONSOLIDA_INSS_COMPL');
        
        
        if ($this->_nrOcorrenciaCalculo > 1 && $consolidaINSScalculo == 1) {
            $recalculo = $this->verificaEventosRecalculo();
            if(!$recalculo){
                MsgCalculo::getInstance()->adicionaMensagem('Vinculo não possui eventos do tipo recalculo', MsgCalculo::ERRO);
                return;
            }
        }
        
        $this->_controleLiquidoParcial();
        
        $this->_inicializaVariaveisRegra();

        $this->_calculo_regra->geraVariaveis();
        
        $this->_calculo_regra->executaPontoRegra(
            $this->_idDebug, FpaPtregraEntity::PTREGRA_INICIO, $this->_nr_tipo_calculo,$this->_itens_calculo_principal
        );
        
        if($this->_idConsolidaInss == 'S'){
             $this->_consolidaInssOcor();
             $this->_consolidaIrrfOcor();
        }
        
        $this->_verificaTrabalhoEfetivo($vinculo);
        
        MemoriaCalculo::getInstance()->criaCabecalho($this->entityManager, $this->hcmParameters, $this->_vinculo,$this->_nr_tipo_calculo,$this->_sit_func_resumo,$this->_salario,$this->_dias_uteis,$this->_dias_rsr,
            $this->_mes_competencia, $this->_dt_pagamento, $this->_nrOcorrenciaCalculo,$this->_obj_tipo_calculo->getNmtipocalculo(),$this->_gozo_ferias);
        
        $this->_geraReferenciaSitFunc();
        
        $this->_calculaRemuneracaoBase($vinculo, $this->_itens_calculo_principal, $this->_nr_tipo_calculo);
        
        if($this->_idPagaAdicional === 'S'){
            $this->_pagaAdicionalFerias();
        }
        
        if($this->_idDifSal === 'S'){
            $this->_verificaDifSalarial($this->_mes_competencia, $vinculo);
        }

        $this->_demonstraAdiantamento($tp_calc_mov);
        
        $this->_demonstraFerias($tp_calc_mov);
        
        if($this->_idDifFerias === 'S') {
            $this->_verificaDifFerias($tp_calc_mov, $vinculo);
        }
        
        $this->_calculaLicencaRemun();

        $this->_calculaValeTransporte();

        if((int)$this->_sit_func_resumo[GpesitufuncmEntity::SITUACAO_FUNCIONAL_NORMAL]>0 ||
            (int)$this->_sit_func_resumo[GpesitufuncmEntity::SITUACAO_FUNCIONAL_MATERNIDADE]>0)
        {
            $this->_calculaContribuicaoSindical($vinculo);
        }
        
        $this->_analisaSalMaternidade($vinculo, $tp_calc_mov);
        
        $this->_calculo_regra->executaPontoRegra(
            $this->_idDebug, FpaPtregraEntity::PTREGRA_ANTES_CALC_ESPECIFICO, $this->_nr_tipo_calculo,$this->_itens_calculo_principal
        );    
        
        $this->_executaFolha($vinculo);
        
        $this->_calculaSalarioFamilia($vinculo);
        
        $this->_calculaGrupoIncidencias();
        
        $this->_geraPensao();
        
        $this->_calculaTotais();
        
        if (($this->_recalculo === true) || ($this->_nrOcorrenciaCalculo > 1)) {
        
            $difFolhaComp = $this->entityManager->getRepository(Repositories::FPA_VALORREFERENCIAM)->buscavalor($this->_nrorg, $this->_nrOrgPadrao, $this->_mes_competencia, 'ID_DIF_FOLHA_COMP');
                   
            if($difFolhaComp == 1){
                if($this->_nrOcorrenciaCalculo==1){
                    MsgCalculo::getInstance()->adicionaMensagem('Para fazer Rescisao Complementar dentro do mês de competência da Rescisão o Nr de Ocorrencia deve ser maior que '.$this->_nrOcorrenciaCalculo);
                }else{
                    $calculo_folha = FpacalculofolhaEntity::retornaCalculoFolha($this->_nrorg, FpatipomovimentEntity::TP_MOV_MENSAL, 1,
                        $this->_nrOcorrenciaCalculo-1, $this->_mes_competencia, $this->_mes_competencia
                    );
                }
        
                $listaNrCalculo = array();
                foreach ($calculo_folha as $calculo){
                    $listaNrCalculo[] = $calculo->getNrcalculofolha();
                }
                $this->_itens_origem->retornaItensPorVinculo($vinculo->getNrorg(), $vinculo->getNrvinculom(), $listaNrCalculo[0]);
        
                /*Retorna objeto do evento associado ao credito de complemento de salario para gerar o evento de debito de complemento no calculo principal*/
                $eventoAuxCred = $this->_assoc_eventos->retornaEventoAssociado(
                    FpatipoassoceveEntity::ASSOC_COMPLEMENTO_SAL_CR , $this->_eventom, $this->_nrorg
                );
                $recuperavalor=$this->_itens_origem->retornaValorPorEvento($eventoAuxCred, $this->_mes_competencia);
        
                $eventoAuxDeb = $this->_assoc_eventos->retornaEventoAssociado(
                    FpatipoassoceveEntity::ASSOC_COMPLEMENTO_SAL_DB , $this->_eventom, $this->_nrorg
                );
                if ($recuperavalor>0) {
                    MemoriaCalculo::getInstance()->escreveEventos($this->_itens_calculo_principal->getNrcalculofolha(), $eventoAuxDeb,null, $this->_mes_competencia, $recuperavalor,0,null,null,null,100);
                    $this->_itens_calculo_principal->geraEvento($eventoAuxDeb, $this->_mes_competencia, $recuperavalor, 0, 1);
                }
        
                $eventoAux = $this->_assoc_eventos->retornaEventoAssociado(
                    FpatipoassoceveEntity::ASSOC_SALARIO_1 , $this->_eventom, $this->_nrorg
                );
                $recuperareferencia=$this->_itens_origem->retornaReferenciaPorEvento($eventoAux, $this->_mes_competencia);
                MemoriaCalculo::getInstance()->escreveEventos($this->_itens_calculo_principal->getNrcalculofolha(), $eventoAux,null, $this->_mes_competencia,1, $recuperareferencia,null,null,null,100);
                $this->_itens_calculo_principal->geraEvento($eventoAux, $this->_mes_competencia,0, $recuperareferencia,1);
        
                $this->_calculaSalario($this->_itens_calculo_principal);
        
                $eventos_transf = $this->_itens_origem->preparaRecalculoProvEventual( $this->_itens_origem, $this->_eventom);
                for ($i=0; $i < count($eventos_transf); $i++) {
                    $evento_aux = $this->_eventom->retornaEvento($eventos_transf[$i]['NREVENTO']);
                    
                    if ($eventos_transf[$i]['VALOR']>0) {
                        MemoriaCalculo::getInstance()->escreveEventos($this->_itens_calculo_principal->getNrcalculofolha(), $eventoAux,null, $eventos_transf[$i]['DTOCORRENCIA'], $eventos_transf[$i]['VALOR'], $eventos_transf[$i]['REFERENCIA'],null,null,null,100);
                        $this->_itens_calculo_principal->geraEvento($evento_aux, $eventos_transf[$i]['DTOCORRENCIA'],  $eventos_transf[$i]['VALOR'], $eventos_transf[$i]['REFERENCIA'], 1);
                    }
        
                }
        		if ($this->_nr_tipo_calculo !== FpatipocalculoEntity::CALC_FOLHA){
        			$this->_calculaDsrSuplementar($this->_itens_calculo_principal, $vinculo);
        		}
        
        		$this->_calculaValeTransporte();
        
                $this->_calculaFaltas($this->_itens_calculo_principal);
        
                $this->_calculaAdicionais($this->_itens_calculo_principal);
        
                $this->_calculo_regra->executaPontoRegra(
                    $this->_idDebug, FpaPtregraEntity::PTREGRA_APOS_ADICIONAIS, $this->_nr_tipo_calculo, $this->_itens_calculo_principal
                );
                
                $this->_calculaReflexo($this->_itens_calculo_principal, FpatipoincideEntity::INC_REFLEXO_ADIC_NOTURNO, FpatipoassoceveEntity::ASSOC_REFLEXO_ADIC_NOTURNO);
        
                $this->_calculaExtra($this->_itens_calculo_principal);
        
                $this->_calculo_regra->executaPontoRegra(
                    $this->_idDebug, FpaPtregraEntity::PTREGRA_APOS_HORAS_EXTRAS, $this->_nr_tipo_calculo, $this->_itens_calculo_principal
                );
                
                $this->_calculaReflexo($this->_itens_calculo_principal, FpatipoincideEntity::INC_REFLEXO_HORA_EXTRA, FpatipoassoceveEntity::ASSOC_REFLEXO_REP_REMUNERADO);
        
        
                $this->_itens_calculo_principal->calculaDiferencaRecalculo($this->_itens_origem, $this->_eventom, $this->_id_fazDifComp);
            }else {
                $this->_id_fazDifComp='N';
    
                if($this->_id_fazDifComp==FpaformrescisaoEntity::ID_FAZDIFERENCACOMP_NAO) {
                    $this->_itens_calculo_principal->executaRecalculoMensal($this->_itens_calculo_principal, $this->_eventom, $this->_id_fazDifComp);
                }
    
                $this->_executaFolha($vinculo);
                $this->_calculaGrupoIncidencias();
                $this->_calculaTotais();
            }
        }
        
        $this->_calculo_regra->executaPontoRegra(
            $this->_idDebug, FpaPtregraEntity::PTREGRA_FINAL, $this->_nr_tipo_calculo,$this->_itens_calculo_principal
        );
        
        MemoriaCalculo::getInstance()->setMovimentos();
        MemoriaCalculo::getInstance()->setIncidencias();
        
        $arquivoMemoria = MemoriaCalculo::getInstance()->getNmArquivo();
        
        $lista = null;
        
        if($arquivoMemoria){
            $lista['ARQUIVOMEMORIA'] = FPA::getTempDirectory().'/'.$arquivoMemoria;
        }
        
        $msgcalculo = MsgCalculo::getInstance()->getDados();
        
        if ($msgcalculo['IDSTATUS'] === MsgCalculo::ID_COM_ERRO) {
            return $lista;
        }
        if ($this->_simulacao === false) {
            $tp_calc_mov->persisteItensCalculo();
            return $lista;
        } else {
            $lista['SIMULACAO'] = $tp_calc_mov->retornaListaItensCalculo(
                $this->_mes_competencia, $this->_eventom, $vinculo
            );
            //Vinculo
            if(count($lista) > 0){
                $lista['SIMULACAO'][0][] = array(
                    'MESOCORRENCIA' => $vinculo->getNrvinculom(),
                    'TIPOEVENTO' => $vinculo->getNmvinculom(),
                    'NRFILHO' => $vinculo->getNrvinculom(),
                    'NREVENTOM' => 'Matricula: '.$vinculo->getCdmatricula(),
                    'NMEVENTOH' => 'Salario: ' .number_format($this->_salario, 2, ',','.'),
                    'UNIDREFEEVEN' => 'Admissão: '.substr($vinculo->getDtadmissaovinc(), 0, 10),
                    'VRREFFOLHA' => null,
                    'IDEVENDEMON' => null,
                    'VREVENTOFOLHA' => null,
                    'NRORDEIMPRES' => null,
                    'ORDEMTIPOEVENTO' => null,
                    'NRVINCULOM' => null,
                    'NRCALCULOFOLHAVINC' => null
                );    
            }
            return $lista;
        }
    }
}