<?php

namespace HCM\FPA\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;

class CalculoFolhaController extends Crud {

    protected $calculoFolhaService;
    protected $dataSourceGetFormCalculo = 'fpaFormcalculo_getFormCalculo';
    protected $dataSourceSetFormCalculo = 'fpaFormcalculo_setFormCalculo';

    protected $parameterBagColumns = array('P_NRORG', 'P_IDTPFORMCALCULO', 'P_CDOPERADOR', 'P_NRORG_PADRAO');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, \HCM\FPA\Services\CalculoFolhaService $calculoFolhaService){
        parent::__construct($dataSourceManager);
        $this->parameterBag = $parameterBag;
        $this->calculoFolhaService = $calculoFolhaService;
    }

    public function getFormCalculo (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceGetFormCalculo);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $rows = $dataSet->getRows();
        
        foreach($rows as $key => &$row){
            if($row['DSORDEM'] == 'LISTA_VINCULO'){
                $row['NRVINCULOM_LIST'] = explode(",", $row['DSINICIO']);
                foreach($row['NRVINCULOM_LIST'] as $chave => $value)
                    $row['NRVINCULOM_LIST'][$chave] = intval($value);
                
                $row['NMVINCULOM'] = explode(",", $row['NMVINCULOM']);
            }    
            
            if($row['DSORDEM'] == 'LISTA_ESTRUTURA'){
                $row['LIST_NRESTRUTLEGAL'] = explode(",", $row['DSINICIO']);
                foreach($row['LIST_NRESTRUTLEGAL'] as $chave => $value)
                    $row['LIST_NRESTRUTLEGAL'][$chave] = intval($value);
                
                $row['LIST_NMESTRUTLEGAL'] = explode(",", $row['NMESTRUTLEGAL']);
                
                if($row['DSFIM']){
                    $row['LIST_NRESTRUTGEREN'] = explode(",", $row['DSFIM']);
                    foreach($row['LIST_NRESTRUTGEREN'] as $chave => $value)
                        $row['LIST_NRESTRUTGEREN'][$chave] = intval($value);
                    
                    $row['LIST_NMESTRUTGEREN'] = explode(",", $row['NMESTRUTGEREN']);
                }
            }   
            
            if($row['DSORDEM'] === "LISTA_NRESTRUTSIND"){
                if($row['DSFIM']) {
                    $row['LIST_NRESTRUTSIND'] = explode(',',$row['DSFIM']);
                    foreach($row['LIST_NRESTRUTSIND'] as $chave => $value)
                        $row['LIST_NRESTRUTSIND'][$chave] = intval($value);
                        
                    $row['LIST_NMESTRUTSIND'] = explode(",", $row['NMESTRUTSIND']);
                }
            }
            
            if($row['DSORDEM'] == 'ESTRUT_ALFABETICA_TPVINC'){
                $row['LIST_NRESTRUTLEGAL'] = explode(',',$row['DSINICIO']);
                foreach($row['LIST_NRESTRUTLEGAL'] as $chave => $value){
                    $row['LIST_NRESTRUTLEGAL'][$chave] = intval($value);
                }
                $row['DSINICIO'] = $row['LIST_NRESTRUTLEGAL'];
                $row['LIST_NMESTRUTLEGAL'] = explode(",", $row['NMESTRUTLEGAL']);
                
                $row['LIST_NRESTRUTGEREN'] = explode(',',$row['DSFIM']);
                foreach($row['LIST_NRESTRUTGEREN'] as $chave => $value){
                    $row['LIST_NRESTRUTGEREN'][$chave] = intval($value);
                }
                
                if($row['DSFIM']){
                    $row['LIST_NRESTRUTGEREN'] = explode(',', $row['DSFIM']);
                    foreach($row['LIST_NRESTRUTGEREN'] as $chave => $value){
                        $row['LIST_NRESTRUTGEREN'][$chave] = intval($value);
                    }
                    $row['DSFIM'] = $row['LIST_NRESTRUTGEREN'];
                    $row['LIST_NMESTRUTGEREN'] = explode(",", $row['NMESTRUTGEREN']);
                }
                
                $row['NMVINCULOM_INI'] = $row['DSINICIO2'];
                $row['NMVINCULOM_FIM'] = $row['DSFIM2'];
                
                $row['NRTIPOVINCULO'] = explode(",", $row['DSINICIO3']);
                foreach($row['NRTIPOVINCULO'] as $chave => $value){
                    $row['NRTIPOVINCULO'][$chave] = intval($value);
                }
                $row['DSINICIO3'] = $row['NRTIPOVINCULO'];
                $row['NMTIPOVINCULO'] = explode(",", $row['NMTIPOVINCULO']);
            }
        }
        $response->addDataSet(new DataSet($this->dataSourceGetFormCalculo, $rows));
    }

    public function setFormCalculo (DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceSetFormCalculo;
        try{
            foreach($request->getDataSet()->getRows() as $row) {
                
                if($row['__is_new'] === true){
                    $row['DTINCLUSAO'] = $row['DTATUAL'];
                    $row['CDOPERINCLUSAO'] = $row['CDOPERADOR_LOG'];
                    $row['NRORGINCLUSAO'] = $row['NRORGTRAB'];
                    $row['DTULTATU'] = $row['DTATUAL'];
                    $row['CDOPERULTATU'] = $row['CDOPERADOR_LOG'];
                    $row['NRORGULTATU'] = $row['NRORGTRAB'];
                    $row['NRFORMCALCULO'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORGTRAB'], 'FPA_FORMCALCULO', 1, 12, true);
                } else {
                    $row['DTULTATU'] = $row['DTATUAL'];
                    $row['CDOPERULTATU'] = $row['CDOPERADOR_LOG'];
                    $row['NRORGULTATU'] = $row['NRORGTRAB'];
                }
                
                if(!array_key_exists('DSINICIO', $row)){
                    $row['DSINICIO'] = null;
                }
 
                if(!array_key_exists('DSFIM', $row)){
                    $row['DSFIM'] = null;
                }
                
                // define os campos de descrição de início e fim com base no critério de seleção passado pela row
                if($row['DSORDEM'] === "LISTA_VINCULO"){
                    if(is_array($row['DSINICIO'])){
                        $row['DSINICIO'] = implode(',',$row['DSINICIO']);
                    }
                }
                
                if($row['DSORDEM'] === "LISTA_ESTRUTURA"){
                    if(is_array($row['DSINICIO'])){
                        $row['DSINICIO'] = implode(',',$row['DSINICIO']);
                    }
                    if(count($row['DSFIM']) > 0 && is_array($row['DSFIM'])){
                        $row['DSFIM'] = implode(',',$row['DSFIM']);
                    }else{
                        if(is_array($row['DSFIM'])){
                            $row['DSFIM'] = null;
                        }
                    }
                }
                
                if($row['DSORDEM'] === "LISTA_NRESTRUTSIND"){
                    if(count($row['DSFIM']) > 0 && is_array($row['DSFIM'])){
                        $row['DSFIM'] = implode(',',$row['DSFIM']);
                    }else{
                        if(is_array($row['DSFIM'])){
                            $row['DSFIM'] = null;
                        }
                    }
                }
                
                if($row['DSORDEM'] === "ESTRUT_ALFABETICA_TPVINC"){
                    if(is_array($row['DSINICIO'])){
                        $row['DSINICIO'] = implode(',',$row['DSINICIO']);
                    }
                    if(is_array($row['DSFIM'])){
                        $row['DSFIM'] = implode(',',$row['DSFIM']);
                    }
                    if(is_array($row['DSINICIO3'])){
                        $row['DSINICIO3'] = implode(',',$row['DSINICIO3']);
                    }
                }
                
                $row['NRORG'] = $row['NRORGTRAB'];
                $row['CDOPERADOR'] = $row['CDOPERADOR_LOG'];
                
                if(!array_key_exists('IDDIFSAL', $row)){
                    $row['IDDIFSAL'] = null;
                }
                
                if(!array_key_exists('IDDIFFERIAS', $row)){
                    $row['IDDIFFERIAS'] = null;
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteFormCalculo (DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $this->dataSourceName = $this->dataSourceSetFormCalculo;
            parent::delete($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }

    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        
        return $newFilterCriteria;
    }
}