<?php

namespace HCM\FPA\ObrigacaoLegal\Demonstrativo_Eletronico\itau;

//use \HCM\FPA\ObrigacaoLegal\Demonstrativo_Eletronico\HoleriteUtil;
use HCM\Util\FPA;

class HoleriteItau extends HoleriteItauRepository {
    /*Mensagens*/
    protected $dsstatus = array(
        'erro'   => "O Holerite do banco Itaú não foi gerado. Por favor, verifique as mensagens geradas.",
        'sucess' => "Holerite gerado com sucesso."
        );
    private $nomeArquivo = 'holerite.txt';
    
    /*Parâmetros de tela*/
    private $nrorg;
    private $nrestrutrespons;
    private $nreventom;
    private $nrtipomoviment;
    private $dtpagamento;
    private $dsordem;
    private $dsinicio;
    private $dsfim;
    
    /*Sistema*/
    private $dtmescompetenc;
    private $nrtpmodalidcal;
    private $connection;
    
    /*Auxiliar*/
    private $qdeLinhas;
    private $vrTotal;
    
    //Array dos Registros prontos
    private $REG0;
    private $REG1;
    private $REG3;
    private $REG5;
    private $REG9;
    
    
    public function __construct($dataSet, $hcmParameters, $connection){
        $this->nrorg           = $dataSet['NRORG'];
        $this->nrestrutrespons = $dataSet['NRESTRUTRESPONS'];
        $this->nreventom       = $dataSet['NREVENTOM'];
        $this->nrtipomoviment  = $dataSet['NRTIPOMOVIMENT'];
        $this->dtpagamento     = $dataSet['DTPAGAMENTO'];
        $this->dsordem         = $dataSet['DSORDEM'];
        $this->dsinicio        = $dataSet['DSINICIO'];
        $this->dsfim           = $dataSet['DSFIM'];
        
        $this->dtmescompetenc  = substr($hcmParameters->getDtCompetencia(), 0, 10);
        $this->nrtpmodalidcal  = $hcmParameters->getTipoModalidadeCalculo();
        $this->connection      = $connection;
        
        $this->qdeLinhas = 0;
        $this->vrTotal   = 0;
        
        $this->REG0 = null;
        $this->REG1 = null;
        $this->REG3 = null;
        $this->REG5 = null;
        $this->REG9 = null;
    }
    
    
    public function makeHolerite() {
        //Reg0
        $mensagens = $this->makeReg0();
        if(is_array($mensagens))
            return array('MENSAGENS' => $mensagens, 'DSSTATUS' => $this->dsstatus['erro']);
        
        //Reg1
        $mensagens = $this->makeReg1();
        if(is_array($mensagens))
            return array('MENSAGENS' => $mensagens, 'DSSTATUS' => $this->dsstatus['erro']);

        //Reg3
        $mensagens = $this->makeReg3();
        if(is_array($mensagens))
            return array('MENSAGENS' => $mensagens, 'DSSTATUS' => $this->dsstatus['erro']);

        //Reg5
        $this->makeReg5(1, 2); //todo
        
        //Reg9
        $this->makeReg9(66); //todo
        
        //Monta o TXT
        $buff_0 = $this->arrayToString($this->REG0, ''    );
        $buff_0 = $this->arrayToString($buff_0    , "\r\n");
        
        $buff_1 = $this->arrayToString($this->REG1, ''    );
        $buff_1 = $this->arrayToString($buff_1    , "\r\n");
        
        $buff_3 = $this->arrayToString($this->REG3, ''    );
        //$buff_3 = $this->arrayToString($buff_3    , '\r\n');
        $buff_3 = implode("\r\n", $buff_3);
        
        $buff_5 = $this->arrayToString($this->REG5, ''    );
        $buff_5 = $this->arrayToString($buff_5    , "\r\n");
        
        $buff_9 = $this->arrayToString($this->REG9, ''    );
        $buff_9 = $this->arrayToString($buff_9    , "\r\n");
        
        //Cria o arquivo
        FPA::escreveArquivo($this->nomeArquivo, $buff_0."\r\n", false);
        FPA::escreveArquivo($this->nomeArquivo, $buff_1."\r\n", false);
        FPA::escreveArquivo($this->nomeArquivo, $buff_3."\r\n", false);
        FPA::escreveArquivo($this->nomeArquivo, $buff_5."\r\n", false);
        FPA::escreveArquivo($this->nomeArquivo, $buff_9."\r\n", false);

        //Se não houver mensagens este retorna sucesso
        //if(!is_array($mensagens))
            return array('DSSTATUS'           => $this->dsstatus['sucess'],
                         'ARQUIVO_ELETRONICO' => FPA::getTempDirectory().'/'.$this->nomeArquivo);
    }
    
    /*Registros*/
    private function makeReg0() {
        $params = array(
            'P_NRORG'          => $this->nrorg,
            'P_DTMESCOMPETENC' => $this->dtmescompetenc,
            'P_NRESTRUTURAM'   => $this->nrestrutrespons
            );
        $reg0 = $this->connection->fetchAll($this->queryReg0, $params);
        
        /*Verificação básica*/
        if(count($reg0) == 0)
            return array( array('NRVINCULOM' => null, 'NRMSGCALC' => null, 'DSDESCRICAO' => "Verifique a conta corente e/ou o histórico da estrutura responsável informada ($this->nrestrutrespons).", 'IDTIPOMSG' => 'Falta parametrização') );
        
        /*Verificação detalhada*/
        $mensagens = $this->testMissingRequired($reg0[0], $this->requireds); //Apenas da linha 0
        if(count($mensagens) > 0) {
            foreach($mensagens as $m)
                $return[] = array('NRVINCULOM' => null, 'NRMSGCALC' => null, 'DSDESCRICAO' => $m, 'IDTIPOMSG' => 'Campo obrigatório vazio');
            return $return;
        }

        /*Concatenar os registros constantes e os registros da query*/
        $reg0 = $this->concatArraysInArray($this->layoutReg0, $this->constReg0, $reg0);
        
        /*Formata o registro*/
        $reg0 = $this->formatStringArray($reg0, $this->layoutReg0);
        
        /*Salva o registro*/
        $this->REG0 = $reg0;
        
        $this->qdeLinhas++;
        
        return null;
    }
    
    private function makeReg1() {
        $params = array(
            'P_NRORG'          => $this->nrorg,
            'P_DTMESCOMPETENC' => $this->dtmescompetenc,
            'P_NRESTRUTURAM'   => $this->nrestrutrespons,
            'P_EVENTO'         => $this->nreventom,
            'P_TIPOMOVIMENT'   => $this->nrtipomoviment
            );
        $reg1 = $this->connection->fetchAll($this->queryReg1, $params);
        
        /*Verificação detalhada*/
        $mensagens = $this->testMissingRequired($reg1[0], $this->requireds); //Apenas da linha 0
        if(count($mensagens) > 0) {
            foreach($mensagens as $m)
                $return[] = array('NRVINCULOM' => null, 'NRMSGCALC' => null, 'DSDESCRICAO' => $m, 'IDTIPOMSG' => 'Campo obrigatório vazio');
            return $return;
        }
        
        /*Concatenar os registros constantes e os registros da query*/
        $reg1 = $this->concatArraysInArray($this->layoutReg1, $this->constReg1, $reg1);
        
        /*Formata o registro*/
        $reg1 = $this->formatStringArray($reg1, $this->layoutReg1);

        /*Salva o registro*/
        $this->REG1 = $reg1;
        
        $this->qdeLinhas++;
        
        return null;
    }
    
    private function makeReg3() {
        $this->REG3 = array();
        
        /*Registros constantes do formulário*/
        $this->constReg3['DATPAGTO'] = str_replace('/', '', $this->dtpagamento);
        $this->constReg3['COMPETEN'] = str_replace('/', '', substr($this->dtmescompetenc, 3, 7));
        
        $params = array(
            'P_NRORG'          => $this->nrorg,
            'P_DTMESCOMPETENC' => $this->dtmescompetenc,
            'P_NRTIPOMOVIMENT' => $this->nrtipomoviment,
            'P_NRTPMODALIDCAL' => $this->nrtpmodalidcal,
            'P_NREVENTOM'      => $this->nreventom
            );
        $params = $this->setParamsDsordem($params, $this->dsordem, $this->dsinicio, $this->dsfim);
        $reg3AD = $this->connection->fetchAll($this->queryReg3AD, $params);
        
        /*Verificação detalhada*/
        $return = array();
        foreach($reg3AD as $reg) {
            $mensagens = $this->testMissingRequired($reg, $this->requireds);
            if(count($mensagens) > 0)
                foreach($mensagens as $m)
                    $return[] = array('NRVINCULOM' => $reg['NRVINCULOM'], 'NRMSGCALC' => null, 'DSDESCRICAO' => $m, 'IDTIPOMSG' => 'Campo obrigatório vazio');
        }
        
        //retorno de mensagens de erro
        if(count($return)>0)
            return $return;
        
        //Monta reg3A e reg3D
        $A = array();
        $D = array();
        $A = $this->concatArraysInArray($this->layoutReg3A, $this->constReg3, $reg3AD); //Concatenar os registros constantes e os registros da query
        $D = $this->concatArraysInArray($this->layoutReg3D, $this->constReg3, $reg3AD);
        $A = $this->formatStringArray($A, $this->layoutReg3A);                          //Formata o registro
        $D = $this->formatStringArray($D, $this->layoutReg3D);
        foreach($A as $k => $a) {
            $this->qdeLinhas += 2; //Linha A e D
            $this->vrTotal += $reg3AD[$k]['VRLIQ']; //Totalizador
            $this->REG3[] = $a;
            $this->REG3[] = $D[$k];
            $E = $this->makeReg3E($reg3AD[$k]['NRCALCULOFOLHA'], $reg3AD[$k]['NRVINCULOM'], $reg3AD[$k]['NROREGIS']);
            foreach($E as $e) {
                $this->qdeLinhas++;
                $this->REG3[] = $e;
            }
        }

        return null;
    }
    
    private function makeReg3E($nrcalculofolha, $nrvinculom, $nroregis) {
        $params = array(
            'P_NRORG'          => $this->nrorg,
            'P_DTMESCOMPETENC' => $this->dtmescompetenc,
            'P_NRCALCULOFOLHA' => $nrcalculofolha,
            'P_NRVINCULOM'     => $nrvinculom
            );
        $result = $this->connection->fetchAll($this->queryReg3E, $params);
        $reg3E = array();
        
        $i   = -1; //índice do registro montado (4 colunas)
        $old = 10; //Para verificar 'linha cheia' (cabem 4 colunas)
        foreach($result as $reg) {
            $n = $reg['N'];
            if($n <= $old) {
                $i++;
                for($j = 1; $j <= 4; ++$j) { //preenche todas as 4 colunas, inclusive as não usadas
                    $reg3E[$i]['DESCRI'.$j] = ' ';
                    $reg3E[$i]['VALOR'.$j ] = '0';
                }
            }
            $old = $n;
            $reg3E[$i]['NROREGIS' ] = $nroregis;
            $reg3E[$i]['MOVIMENT' ] = $reg['MOVIMENT'];
            $reg3E[$i]['DESCRI'.$n] = $reg['DESCRI'];
            $reg3E[$i]['VALOR'.$n ] = $reg['VALOR' ];
        }
        
        /*Concatenar os registros constantes e os registros da query*/
        $reg3E = $this->concatArraysInArray($this->layoutReg3E, $this->constReg3, $reg3E);
        
        /*Formata o registro*/
        $reg3E = $this->formatStringArray($reg3E, $this->layoutReg3E);

        return $reg3E;
    }
    
    private function makeReg5($vrTotal) {
        $reg5[0] = array(
            'TOTALREG' => $this->qdeLinhas,
            'TOTALVAL' => $this->vrTotal
            );
        
        /*Concatenar os registros constantes e o registro de totalizador*/
        $reg5 = $this->concatArraysInArray($this->layoutReg5, $this->constReg5, $reg5);
        
        /*Formata o registro*/
        $reg5 = $this->formatStringArray($reg5, $this->layoutReg5);
        
        /*Salava o registro*/
        $this->REG5 = $reg5;
    }
    
    private function makeReg9() {
        $this->qdeLinhas++;
        $reg9[0] = array(
            'QDEREGIS' => $this->qdeLinhas + 1
            );
        
        /*Concatenar os registros constantes e o registro de totalizador*/
        $reg9 = $this->concatArraysInArray($this->layoutReg9, $this->constReg9, $reg9);
        
        /*Formata o registro*/
        $reg9 = $this->formatStringArray($reg9, $this->layoutReg9);
        
        /*Salava o registro*/
        $this->REG9 = $reg9;
    }
    
}