<?php

namespace HCM\Geral\Services;

use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
//use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Geral\Repository\VinculoRepository;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Response\Error;

use HCM\Util\PhpWord\TemplateProcessorHCM;

class EmissaoDocService {

    protected $entityManager;
    protected $hcmParameters;
    protected $vinculoRepository;
    
    protected $mnemonicsDoc      = array();
    protected $mnemonicsSystem   = array();
    protected $repositoriesInUse = array();
    protected $listDocsFiles     = array();
    
    private $urlMidiaServer;
    
    
    
    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters, VinculoRepository $vinculoRepository) {
        $this->entityManager     = $entityManager;
        $this->hcmParameters     = $hcmParameters;
        $this->vinculoRepository = $vinculoRepository;
    }
    
    
    /**
     * FUNÇÃO PRINCIPAL
     */
    public function emitirDocPadrao($nrorg, $urlMidiaServer, $dtmescompetencia, $dtmescompetenciafim, $dspath, $nmdocumento, $nrvinculom_list, $nrestrutlegal_list, $nrestrutgeren_list, $nrestrutsind, $dtiniadmissao, $dtfimadmissao, $dtinirescisao, $dtfimrescisao) {
        
        error_reporting(E_ERROR); //todo por causa do phpword. gerar warnings que quebrar o retorno. rever isto
        $this->urlMidiaServer = $urlMidiaServer;
        
        //Adquiri a lista dos mnemônicos existêntes no sistema
        $this->getMnemonicsSystem();
        
        //Adquiri o documento
        $pathDocName = $this->getDocModelo($dspath, $nmdocumento, $nrorg);
        
        //Adquiri lista de todos mnemônicos usados no documento //todo passar para uma função?
        $templateProcessor = new TemplateProcessorHCM($pathDocName); //XML do documento
        $this->mnemonicsDoc = $templateProcessor->getMnemonics();
        if(count($this->mnemonicsDoc) == 0) //Verifica se o documento possui pelo menos um mnemônico
            throw new \Exception("HCM10010_GER_EMISSAODOC_NO_MNEMONIC");
        unset($templateProcessor); //Destroi a instância
        
        //Monta uma lista dos repositories usados pelo documento
        $this->getRepositoriesInUse();
        
        //Monta uma lista de vínculos de acordo com o formulário da tela
        $vinculos = $this->vinculoRepository->retornaVinculosAtivosDoc($nrorg, $dtmescompetencia, $nrestrutlegal_list, $nrestrutgeren_list, $nrestrutsind, $nrvinculom_list, $dtiniadmissao, $dtfimadmissao, $dtinirescisao, $dtfimrescisao);
        if(count($vinculos) == 0) //Verifica se os filtros do formulário trouzeram pelo um ou mais vínculos
            throw new \Exception("HCM10010_GER_EMISSAODOC_NO_RESULT");
        
        //LOOP - Percorre todos os vínculos retornados pela seleção do formulário
        foreach($vinculos as $vinculo) {
            
            //Usa o array de mnemônicos do documento (mnmonicsDoc) para popular seus valores
            $this->getMnemonicData($vinculo, $dtmescompetencia);

            //Gera um arquivo novo Doc com os mnemônicos substituídos e armazena a lista
            $this->listDocsFiles[] = $this->gerarNovoDoc($pathDocName, $nrorg, $nmdocumento, $vinculo['NMVINCULOM']);

        }
        
        //todo se a lista de arquivos conter mais de um arquivo acionar a função de zip
        
        return $this->listDocsFiles;
    }
    
    
    
    /**
     * Gera um novo documento .doc ou .docx com as substituições dos menumônicos.
     * Retorna este documento em string.
     */
    private function gerarNovoDoc($pathDocName, $nrorg, $nmdocumento, $nmvinculom) {
        
        //XML do documento - Instância volátil, apenas dentro desta função
        $templateProcessor = new TemplateProcessorHCM($pathDocName);
        
        //Percorre todos o mnemônicos em uso pelo docuemnto
        foreach($this->mnemonicsDoc as $mnemonic => $valueData) {
            $templateProcessor->setValue($mnemonic, $valueData);
        }
        
        /*//Define local e nome do novo documento
        $targetFileName = explode('.', $nmdocumento); //Separa o nome da extensão
        if(count($targetFileName) != 2) //Verifica se o nome possui apenas um ponto '.'
            throw new \Exception("HCM10010_GER_EMISSAODOC_NOT_ONE_DOT");
        $targetFileName = $this->hcmParameters->getTempDirectory().'/'.$nrorg.'/'.$targetFileName[0].'_-_'.$nmvinculom.'.'.$targetFileName[1];

        //Salva o templete como documento doc / docx
        $originFileName = $templateProcessor->save();
        
        //Ajuste do nome e movimentação entre diretórios do arquivo final
        if(!rename($originFileName, $targetFileName))
            throw new \Exception("Não foi possivel mover '$originFileName' para '$targetFileName'.");
        
        return $targetFileName;*/
        
        //Define local e nome do novo documento
        $targetFileName = explode('.', $nmdocumento); //Separa o nome da extensão
        if(count($targetFileName) != 2) //Verifica se o nome possui apenas um ponto '.'
            throw new \Exception("HCM10010_GER_EMISSAODOC_NOT_ONE_DOT");
        $targetFileName = $this->hcmParameters->getTempDirectory().'/'.$nrorg.'/'.$targetFileName[0].'_-_'.$nmvinculom.'.'.$targetFileName[1];
        
        //Salva o templete com o novo nome
        $templateProcessor->saveAs($targetFileName);
        
        unset($templateProcessor); //Destroi a instância
        
        //todo um isFile e uma throw??
        
        return $targetFileName;
    }
    
    
    
    /**
     * Baixa o arquivo do file server para o temporário e retorna seu nome completo com o caminho.
     * Retorna mensagem de erro se necessário. Se não carregar este retorna falso.
     */
    public function getDocModelo($path, $fileName, $nrorg) {
        
        if($path == null)
            throw new \Exception("HCM10010_GER_EMISSAODOC_NO_DSPTAH");
        
        //Local do destino do arquivo baixado.
        $filePath = $this->hcmParameters->getTempDirectory().'/'.$nrorg.'/'.$fileName;

        //Conteúdo do arquivo do file server
        $fileContents = file_get_contents($this->urlMidiaServer.$path);

        //Salva no servidor o arquivo
        if($fileContents) { //Verifica se existe arquivo
            file_put_contents($filePath, $fileContents);
        } else {
           throw new \Exception("HCM10010_GER_EMISSAODOC_FILECONTENTS_FAIL");
        }
        
        return $filePath;
    }
    
    
    
    /**
     * Ajusta o nome do novo docx gerado pelo TemplateProcessor (PhpWord)
     */
    public function renameTemplate($template, $fileName, $nrorg) {
        
        //Local do destino do arquivo temporário template
        $filePath = $this->hcmParameters->getTempDirectory().'/'.$nrorg.'/'.$fileName;
        unlink($filePath); //Apaga temporário anterior
        rename($template, $filePath);
        
        return $filePath; //Novo nome e caminho do template (docx)
    }
    
    
    
    /**
     * Consulta e monta na memória o repositório dos mnemônicos
     */
    private function getMnemonicsSystem() {
        
        //Limpa/configura o mapeamento dos mnemônicos
        $this->mnemonicsSystem = array();
        
        //Mnemônicos da org padrão
        $objMnemonico = $this->entityManager->getRepository(Repositories::GPE_MNEMONICO)->findBy(array('nrorg' => $this->hcmParameters->getNrorgPadrao()));
        foreach($objMnemonico as $obj) {
            $this->mnemonicsSystem[$obj->getCdmnemonico()] =
                array(
                    'dsrepositorio' => $obj->getDsrepositorio(),
                    'dsatributo'    => $obj->getDsatributo(),
                    'dsparamaux'    => $obj->getDsparamaux()
                ); //todo futuramente colocar a entidade adicionar o dsparamauxiliar
        }
        
        //Mnemônicos da org do cliente - Propositalmente assim para que mnemônicos do cliente sobrescrevam os mnemônicos padrão
        $objMnemonico = $this->entityManager->getRepository(Repositories::GPE_MNEMONICO)->findBy(array('nrorg' => $this->hcmParameters->getNrorgTrabalho()));
        foreach($objMnemonico as $obj) {
            $this->mnemonicsSystem[$obj->getCdmnemonico()] =
                array(
                    'dsrepositorio' => $obj->getDsrepositorio(),
                    'dsatributo'    => $obj->getDsatributo(),
                    'dsparamaux'    => $obj->getDsparamaux()
                ); //todo futuramente colocar a entidade adicionar o dsparamauxiliar
        }
        
        unset($objMnemonico);
    }
    
    
    
    /**
     * Monta uma lista dos repositories/mnemônicos usados pelo documento.
     * O índice é o nome do repositorio. Armazena dados para a chamada deste repositorio.
     */
    private function getRepositoriesInUse() {
        
        $this->repositoriesInUse = array();
        
        //Percorre todos o mnemônicos usados pelo documento
        foreach($this->mnemonicsDoc as $mnemonic => $v) {
            
            //Apenas mnemônicos que existam no sistema
            if(isset($this->mnemonicsSystem[$mnemonic])) {
                
                $dsrepositorio = $this->mnemonicsSystem[$mnemonic]['dsrepositorio'];
                $dsatributo    = $this->mnemonicsSystem[$mnemonic]['dsatributo'];
                $dsparamaux    = $this->mnemonicsSystem[$mnemonic]['dsparamaux']; //todo entidade
                
                //Cria uma entrada de indice <nome do repositório>
                if(!isset($this->repositoriesInUse[$dsrepositorio])) {
                    $this->repositoriesInUse[$dsrepositorio] = array();
                }
                
                //Cria um entrada de indice <nome do repositório> / <nome mnemônico>
                $this->repositoriesInUse[$dsrepositorio][$mnemonic] =
                    array(
                        'dsatributo' => $dsatributo,
                        'dsparamaux' => $dsparamaux
                    );
            } /*else { //todo lançar mensagens de mnemônicos não cadastrados? }*/
        }
    }
    
    
    
    /**
     * Popula o array de Mnemônicos do Documento com os valores correspondnetes
     * Usa o array de repositorios em uso para acionar os repositórios correspondnetes.
     * Os valores conseguidos dos repositórios são armazenados no array de mnemônicos dos documentos.
     */
    private function getMnemonicData($params, $dtmescompetencia) {
        
        //Parâmetros padrão para as entidades
        $paramsPadrao['nrorg']          = $this->hcmParameters->getNrorgTrabalho();
        $paramsPadrao['nrorgpadrao']    = $this->hcmParameters->getNrorgPadrao();
        $paramsPadrao['dtmescompetenc'] = $dtmescompetencia;
        
        //Concatenando parâmetros padrão e parâmetros do vínculo
        $params = array_merge($paramsPadrao, $params);

        //Adquiri os dados dos repositorios usados no documento de acordo com o vínculo em questão
        foreach($this->repositoriesInUse as $repository => $repositoryContent) {

            //todo parâmetro auxiliar //$params['DSPARAMAUX']
            
            //Dados do repositório
            $data = $this->entityManager->getRepository(Repositories::GPE_MNEMONICO)->$repository($params);

            //Percorre o dado do indice <nome do repositório>
            foreach($repositoryContent as $mnemonic => $mnemonicContent) {
                
                //Aquisiação do nome do campo corrspondente ao mnemônico e o dado (valor)
                $rowName = $mnemonicContent['dsatributo'];
                $value   = $data[$rowName];
                
                //todo formatação do dado (valor) do campo
                if(substr($rowName, 0, 2) == 'DT') { //TEMPORÁRIO!!!!
                    $value = substr($value, 0, 10);
                }
                
                //Usa o array dos mnemônicos do documento para preencher os dados
                $this->mnemonicsDoc[$mnemonic] = $value;
            }
        }
    }
}