<?php
/**
 * Created by PhpStorm.
 * User: felipesantos
 * Date: 30/06/2016
 * Time: 17:30
 */

namespace HCM\Geral\Services;

use HCM\Util\DBWriter;
use HCM\Util\HCMParameters;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\Events\PostDispatch\Listener;
use Zeedhi\Framework\HTTP\Kernel;
use Zeedhi\Framework\Session\Session;
use Doctrine\ORM\EntityManager;

class ResponseListener extends Listener {

    const STATE_INSERT = 'I';
    const STATE_UPDATE = 'U';
    const STATE_DELETE = 'D';

    /**
     * @var \Zeedhi\Framework\HTTP\Kernel Kernel
     */
    private $httpKernel;
    private $sessionManager;
    private $entityManager;
    private $hcmParameters;
    private $operadorService;

    private $logRoutes;

    public function __construct(Kernel $kernel, Session $sessionManager, EntityManager $entityManager, HCMParameters $hcmParameters, OperadorService $operadorService){
        $this->httpKernel = $kernel;
        $this->sessionManager = $sessionManager;
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->operadorService = $operadorService;

        $this->logRoutes = $this->readRoutes();
    }
 
    private function readRoutes(){
        $urls = file_get_contents("../config/logRoutesMapping.json");
        return json_decode($urls, true);
    }

    public function postDispatch(Request $request, Response $response)
    {
        try {
            if (in_array($request->getRoutePath(), $this->logRoutes) || $this->existsSubRoutePath($request->getRoutePath(),$this->logRoutes)) {
                if (get_class($request) == 'Zeedhi\Framework\DTO\Request\DataSet' && $response->getStatus() === Response::STATUS_SUCCESS) {
                    $delete = false;
                    foreach ($response->getNotifications() as $notification) {
                        if (strpos($notification->getMessage(),'deleted') !== false) {
                            $delete = true;
                        }
                    }
                    $tablename = null;
                    $pkValues = array();
                    
                    foreach ($response->getDataSets() as $dataset) {
                        $tablename = $dataset->getDataSourceName();
                        foreach ($dataset->getRows() as $row) {
                            $pkValues = is_array($row) ? $row : array($row);
                        }
                    }
                    
                    if (is_null($tablename)) {
                        $tablename = $request->getRoutePath();
                    }
                    
                    foreach ($request->getDataSet()->getRows() as &$row) {
                        if($request->getDataSet()->getDataSourceName() == 'saveUploadFile'){
                            unset($row['DSPATH']);
                        }
                        
                        if(isset($row['__is_new'])){
                            if($row['__is_new'] === true){
                                $this->logInsert($tablename, $pkValues, $row);
                            }
                            else if($row['__is_new'] === false) {
                                if (!$delete) {
                                    $this->logUpdate($tablename, $pkValues, $row);
                                }
                                else {
                                    $this->logDelete($tablename, $pkValues, $row);
                                }
                            }
                        }else{
                            $this->logExecute($tablename, $pkValues, $row);
                        }
                    }
                }
            }
        }catch (Exception $e){
            throw new \Exception($e->getMessage());
        }
    }
    
    protected function existsSubRoutePath($routePath, $routes) {
        try {
            $subRoutePath = substr($routePath,0,strrpos($routePath,'/'));
            if (!empty($subRoutePath)) {
                foreach($routes as $route) {
                    if (strpos($route,$subRoutePath) >= 0) {
                        return true;
                    }
                }
            }
            return false;
        } catch (Exception $e){
            throw new \Exception($e->getMessage());
        }
    }

    protected function getPreparedLogWriter($tablename, $pkValues, $row, $type) {

        $writer = new DBWriter($this->entityManager);

        if (!$writer) {
            throw new \Exception('DBWriter not found exception');
        }

        try{
            $operador = $this->operadorService->factoryOperador($this->hcmParameters->getCdoperador());
            $writer->setUsername($operador['nmoperador']);
            $writer->setTableName($tablename);
            $writer->setType($type);
            $writer->setPkValues($pkValues);
            $writer->setDataDiff($row);
            $writer->setExtendedData(array(
                'CDFILIAL' => '9999',
                'NMFILIAL' => $operador['nmorganizacaotrab'],
                'NRORG' => $operador['nrorgtrab'],
                'SYSID' => 'hcmmobile',
                'HOST' => $_SERVER['HTTP_HOST']
            ));
        }catch(\Exception $e){
            $writer->setUsername("Undefined");
            $writer->setTableName($tablename);
            $writer->setType($type);
            $writer->setPkValues($pkValues);
            $writer->setDataDiff($row);
            $writer->setExtendedData(array(
                'CDFILIAL' => '9999',
                'NMFILIAL' => "Undefined",
                'NRORG' => null,
            ));
        }
        return $writer;
    }

    protected function writeLog($tablename, $pkValues, $row, $description, $type)
    {
        $writer = $this->getPreparedLogWriter($tablename, $pkValues, $row, $type);
        $writer->info($description);
    }

    private function logInsert($tablename, $pkValues, $row) {
        $description = 'On INSERT auto log.';
        $this->writeLog($tablename, $pkValues, $row, $description, self::STATE_INSERT);
    }

    private function logUpdate($tablename, $pkValues, $row) {
        $description = 'On UPDATE auto log.';
        $this->writeLog($tablename, $pkValues, $row, $description, self::STATE_UPDATE);
    }

    private function logDelete($tablename, $pkValues, $row) {
        $description = 'On DELETE auto log.';
        $this->writeLog($tablename, $pkValues, $row, $description, self::STATE_DELETE);
    }

    private function logExecute($tablename, $pkValues, $row) {
        $description = 'On EXECUTE auto log.';
        $this->writeLog($tablename, $pkValues, $row, $description, self::STATE_INSERT);
    }
}