<?php
namespace Faturamento\Logic\Cda;

/**
 * Description of FiltroRCDA
 *
 * @author ingridmoreira
 */
class FiltroRCDA {
    
    public static function getDadosOperadorFiltroRcda() {
        $ambiente = \Geral\Util\Ambiente::getInstancia();
        
        /**
         * Código do funcionário relacionado ao operador logado 
         */
        $cdfunc = $ambiente->getOperador()->getFunc()->getCdfunc(); 
        $nrorg = $ambiente->getNrorg();

        /** 
         * Código do tipo de admissão do funcionário relacionado ao operador 
         * logado.
         */
        $cdtpadmissao = $ambiente->getOperador()->getFunc()->getCdtpadmissao();
        
        /**
         * Verifica o período atual de acordo com a data de hoje
         */
        $dtperiodo = self::getPeriodoapurapontoAtual($nrorg, $cdtpadmissao);
        $dtinicial = $dtperiodo['dtinicial'];
        $dtfinal = $dtperiodo['dtfinal'];
        
        /* Monta o array contendo os valores que serão retornados 
         */
        $dadosfiltrocda = array(
            'cdfunc'          => $cdfunc,
            'dtinicial' => $dtinicial,
            'cdtpadmissao'    => $cdtpadmissao,
            'dtfinal'       => $dtfinal
        );
        
        return $dadosfiltrocda;

  }
   
    public static function getPeriodoapurapontoAtual($nrorg, $cdtpadmissao) {
        $dtatual = \Geral\Util\DateUtil::subtraiIntervalo(\Geral\Util\DateUtil::getDataAtual(false), 1);
        $stdtatual  = \Geral\Util\DateUtil::getAno($dtatual);
        $stdtatual .= \Geral\Util\DateUtil::getMes($dtatual);
        $stdtatual .= \Geral\Util\DateUtil::getDia($dtatual);
        
        $where  = "(CDTPADMISSAO = ?) AND (TO_DATE(?, 'YYYYMMDD') BETWEEN DTINICIALAPURAPONTO AND DTFINALAPURAPONTO) AND (NRORG = ?)";
        $params = array($cdtpadmissao, $stdtatual, $nrorg);
        
        $fpa_periodoapuraponto = \Folha\Entities\Fpaperiodoapuraponto::getObjectByWhere($where, $params);
        $dtinicial = $fpa_periodoapuraponto->getDtinicialapuraponto();
        $dtfinal   = $fpa_periodoapuraponto->getDtfinalapuraponto();
        $dtinicial = $dtinicial->format(\Geral\Util\DateUtil::FORMATO_BRASILEIRO);
        $dtfinal   = $dtfinal->format(\Geral\Util\DateUtil::FORMATO_BRASILEIRO);
        $dtperiodo = array(
            'dtinicial' => $dtinicial,
            'dtfinal'   => $dtfinal,
        );
        
        return $dtperiodo;
    }
    
    public static function getTipoAdmissaoFunc($nrorg, $cdempresa, $cdfunc) {
        $func = \Geral\Entities\Funcionario::findOneByCdempresaAndCdfuncAndNrorg($cdempresa, $cdfunc, $nrorg);

        $cdtpadmissao = '';
        if(is_object($func)) {
            $cdtpadmissao = $func->getCdtpadmissao();
        } 
        
        return $cdtpadmissao;
    }
}
